<?php

namespace Drupal\ngrok_drupal;

use Drupal\Core\Routing\UrlGenerator;

/**
 * Class NgrokUrlGenerator.
 *
 * @package Drupal\ngrok_drupal
 */
class NgrokUrlGenerator extends UrlGenerator {

  /**
   * Generates a URL or path for a specific route based on the given parameters.
   *
   * Alter the default generated url by checking if the request header contains
   * a x-original-host header generated by ngrok.io. If so, replace the host
   * by the attributed ngrok.io domain.
   *
   * {@inheritdoc}
   */
  public function generateFromRoute($name, $parameters = array(), $options = array(), $collect_bubbleable_metadata = FALSE) {
    $url = parent::generateFromRoute($name, $parameters, $options, $collect_bubbleable_metadata);

    // Only triggers the check for absolute URL.
    if ($url && array_key_exists('absolute', $options) && $options['absolute'] == TRUE) {

      $urlDefinition = parse_url($url->getGeneratedUrl());
      $request = \Drupal::request();

      // Check if the request has a 'x-original-host' header.
      if ($request->headers->has('x-original-host')) {
        $originalHost = $request->headers->get('x-original-host');

        // Check if the 'x-original-host' header is form ngrok.io.
        if (preg_match('/.*\.ngrok\.io$/', $originalHost) === 1) {

          // Replace the local host with the ngrok.io given address.
          $urlDefinition['host'] = $originalHost;
          $url->setGeneratedUrl($this->unparseUrl($urlDefinition));
        }
      }
    }
    return $url;
  }

  /**
   * Unparse URL array.
   *
   * This is the opposite of the parse_url function.
   *
   * @param array $parsedUrl
   *    The array in the format given by parse_url command.
   *
   * @return string
   *    The generated URL.
   */
  public function unparseUrl(array $parsedUrl) {
    $scheme   = isset($parsedUrl['scheme']) ? $parsedUrl['scheme'] . '://' : '';
    $host     = isset($parsedUrl['host']) ? $parsedUrl['host'] : '';
    $port     = isset($parsedUrl['port']) ? ':' . $parsedUrl['port'] : '';
    $user     = isset($parsedUrl['user']) ? $parsedUrl['user'] : '';
    $pass     = isset($parsedUrl['pass']) ? ':' . $parsedUrl['pass'] : '';
    $pass     = ($user || $pass) ? "$pass@" : '';
    $path     = isset($parsedUrl['path']) ? $parsedUrl['path'] : '';
    $query    = isset($parsedUrl['query']) ? '?' . $parsedUrl['query'] : '';
    $fragment = isset($parsedUrl['fragment']) ? '#' . $parsedUrl['fragment'] : '';
    return "$scheme$user$pass$host$port$path$query$fragment";
  }

}
